#include "test_config_cache.h"

CPPUNIT_TEST_SUITE_REGISTRATION (ConfigCacheTest);

void ConfigCacheTest::setUp() {

  delete _cache_config;
  _tmpfile[0] = '\0';
  strcat(_tmpfile, "/tmp/test_config_cache.XXXXXX");
  _h = mkstemp(_tmpfile);
  if (_h == -1) {
    odlog(ERROR)<<"Error creating file "<<_tmpfile<<" with mkstemp(): "<<strerror(errno)<<std::endl;
  }
  odlog(ERROR)<<"tmp file "<<_tmpfile<<" created"<<std::endl;
  nordugrid_config_loc = _tmpfile;
  central_configuration = true;
}

void ConfigCacheTest::tearDown() {
  // delete all possible files that might have been created
  close(_h);
  remove(_tmpfile);
  delete _cache_config;
}
  
void ConfigCacheTest::testOneCache() {
  std::string buf = "[grid-manager]\ncachedir=\"/tmp/cache\"\ncache_settings=\"2 1\"\ncachesize=\"80 70\"\n";
  CPPUNIT_ASSERT(write(_h, buf.c_str(), buf.length()) != -1);
  CPPUNIT_ASSERT_EQUAL(0, close(_h));
  
  _cache_config = new CacheConfig();
  
  std::list<std::list<std::string> > cache_dirs = _cache_config->getCacheDirs();
  CPPUNIT_ASSERT_EQUAL(1, (int)cache_dirs.size());
  
  std::list<std::list<std::string> >::iterator i = cache_dirs.begin();
  CPPUNIT_ASSERT_EQUAL(3, (int)i->size());
  
  std::list<std::string>::iterator j = i->begin();
  CPPUNIT_ASSERT_EQUAL(std::string("/tmp/cache/data"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string("/tmp/cache/joblinks"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string(""), (*j));
}

void ConfigCacheTest::testTwoCaches() {
  char * buf = "[grid-manager]\ncachedir=\"/tmp/cache\"\ncachedir=\"/tmp2/cache /link2\"\ncache_settings=\"2 1\"\ncachesize=\"80 70\"\n";
  CPPUNIT_ASSERT(write(_h, buf, strlen(buf)) != -1);
  CPPUNIT_ASSERT_EQUAL(0, close(_h));
  
  _cache_config = new CacheConfig();
  std::list<std::list<std::string> > cache_dirs = _cache_config->getCacheDirs();
  CPPUNIT_ASSERT_EQUAL(2, (int)cache_dirs.size());

  std::list<std::list<std::string> >::iterator i = cache_dirs.begin();
  CPPUNIT_ASSERT_EQUAL(3, (int)i->size());
  
  std::list<std::string>::iterator j = i->begin();
  CPPUNIT_ASSERT_EQUAL(std::string("/tmp/cache/data"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string("/tmp/cache/joblinks"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string(""), (*j));
  
  i++;
  CPPUNIT_ASSERT_EQUAL(3, (int)i->size());
  
  j = i->begin();
  CPPUNIT_ASSERT_EQUAL(std::string("/tmp2/cache/data"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string("/tmp2/cache/joblinks"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string("/link2"), (*j));
  
}

void ConfigCacheTest::testSubstitutionCaches() {
  char * buf = "[grid-manager]\ncachedir=\"/tmp/%U/cache\"\ncachedir=\"/tmp2/cache /link2\"\ncache_settings=\"2 1\"\ncachesize=\"80 70\"\ncontrol=\"/tmp/user user1\"\n";
  CPPUNIT_ASSERT(write(_h, buf, strlen(buf)) != -1);
  CPPUNIT_ASSERT_EQUAL(0, close(_h));
  
  _cache_config = new CacheConfig("user1");
  std::list<std::list<std::string> > cache_dirs = _cache_config->getCacheDirs();
  CPPUNIT_ASSERT_EQUAL(2, (int)cache_dirs.size());

  std::list<std::list<std::string> >::iterator i = cache_dirs.begin();
  CPPUNIT_ASSERT_EQUAL(3, (int)i->size());
  
  std::list<std::string>::iterator j = i->begin();
  CPPUNIT_ASSERT_EQUAL(std::string("/tmp/%U/cache/data"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string("/tmp/%U/cache/joblinks"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string(""), (*j));
  
  i++;
  CPPUNIT_ASSERT_EQUAL(3, (int)i->size());
  
  j = i->begin();
  CPPUNIT_ASSERT_EQUAL(std::string("/tmp2/cache/data"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string("/tmp2/cache/joblinks"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string("/link2"), (*j));
}

void ConfigCacheTest::testSubstitutionCachesNoUser() {
  
  char * buf = "[grid-manager]\ncachedir=\"/tmp/%U/cache\"\ncachedir=\"/tmp2/cache /link2\"\ncache_settings=\"2 1\"\ncachesize=\"80 70\"\n";
  CPPUNIT_ASSERT(write(_h, buf, strlen(buf)) != -1);
  CPPUNIT_ASSERT_EQUAL(0, close(_h));

  // create with no username - should ignore the substitution
  _cache_config = new CacheConfig("");
  std::list<std::list<std::string> > cache_dirs = _cache_config->getCacheDirs();
  CPPUNIT_ASSERT_EQUAL(1, (int)cache_dirs.size());

  std::list<std::list<std::string> >::iterator i = cache_dirs.begin();
  CPPUNIT_ASSERT_EQUAL(3, (int)i->size());
  
  std::list<std::string>::iterator j = i->begin();
  CPPUNIT_ASSERT_EQUAL(std::string("/tmp2/cache/data"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string("/tmp2/cache/joblinks"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string("/link2"), (*j));

}

void ConfigCacheTest::testUsers() {
  // settings for for user1
  std::string buf = "[grid-manager]\ncachedir=\"/home/user/cache\"\ncache_settings=\"2 1\"\ncachesize=\"80 70\"\n";
  buf += "control=\"/home/user/control user1 user2 user3\"\n";
  // settings for all others
  buf += "cachedir=\"/tmp/cache\"\ncache_settings=\"1 2\"\ncachesize=\"90 85\"\n";
  buf += "control=\"/data/control .\"\n";
  CPPUNIT_ASSERT(write(_h, buf.c_str(), buf.length()) != -1);
  CPPUNIT_ASSERT_EQUAL(0, close(_h));
  
  // get settings for user1
  _cache_config = new CacheConfig("user1");

  std::list<std::list<std::string> > cache_dirs = _cache_config->getCacheDirs();
  CPPUNIT_ASSERT_EQUAL(1, (int)cache_dirs.size());
  
  std::list<std::list<std::string> >::iterator i = cache_dirs.begin();
  CPPUNIT_ASSERT_EQUAL(3, (int)i->size());
  
  std::list<std::string>::iterator j = i->begin();
  CPPUNIT_ASSERT_EQUAL(std::string("/home/user/cache/data"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string("/home/user/cache/joblinks"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string(""), (*j));

  CPPUNIT_ASSERT_EQUAL(80, _cache_config->getCacheMax());
  CPPUNIT_ASSERT_EQUAL(70, _cache_config->getCacheMin());

  // get settings for user3
  _cache_config = new CacheConfig("user3");

  cache_dirs = _cache_config->getCacheDirs();
  CPPUNIT_ASSERT_EQUAL(1, (int)cache_dirs.size());
  
  i = cache_dirs.begin();
  CPPUNIT_ASSERT_EQUAL(3, (int)i->size());
  
  j = i->begin();
  CPPUNIT_ASSERT_EQUAL(std::string("/home/user/cache/data"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string("/home/user/cache/joblinks"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string(""), (*j));

  CPPUNIT_ASSERT_EQUAL(80, _cache_config->getCacheMax());
  CPPUNIT_ASSERT_EQUAL(70, _cache_config->getCacheMin());

  // get settings for another user
  delete _cache_config;
  _cache_config = new CacheConfig("user99");
  
  cache_dirs = _cache_config->getCacheDirs();
  CPPUNIT_ASSERT_EQUAL(1, (int)cache_dirs.size());
  
  i = cache_dirs.begin();
  CPPUNIT_ASSERT_EQUAL(3, (int)i->size());
  
  j = i->begin();
  CPPUNIT_ASSERT_EQUAL(std::string("/tmp/cache/data"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string("/tmp/cache/joblinks"), (*j));
  j++;
  CPPUNIT_ASSERT_EQUAL(std::string(""), (*j));
  
  CPPUNIT_ASSERT_EQUAL(90, _cache_config->getCacheMax());
  CPPUNIT_ASSERT_EQUAL(85, _cache_config->getCacheMin());

}

void ConfigCacheTest::testConstructorExceptions() {
  // bad cache size params
  char * buf = "[grid-manager]\ncachedir=\"/tmp/cache\"\ncache_settings=\"2 1\"\ncachesize=\"badnumber 70\"\n";
  CPPUNIT_ASSERT(write(_h, buf, strlen(buf)) != -1);
  CPPUNIT_ASSERT_EQUAL(0, close(_h));
  
  CPPUNIT_ASSERT_THROW(new CacheConfig(), CacheConfigException);
  close(_h);
  CPPUNIT_ASSERT_EQUAL(0, remove(_tmpfile));
}
